/*
 *  Usage Bixolon CUPS API
 *
 *  ************* CAUTION **************
 *
 *  You must disable SELinux for using Bixolon API.
 *
 *  Change setting at /etc/sysconfig/selinux file
 *  
 *  SELINUX=disabled
 *
 *  ************************************
 *
 *  Support API function 
 *  - startBxlAPIMode (const char *printerName)
 *  - bxlCmdWriter (const char *buffer, int bufferLen)
 *  - bxlReader (char *buffer, const int timeout)
 *  - finishBxlAPIMode ();
 *  
 *  1. startBxlAPIMode (const char *printerName)
 *  		printerName : Thist name is that you inputted printer name 
 *  		              when you add printer to CUPS system.
 *	    
 *	   - You must use this function at the first time for using Bixolon CUPS API.
 *
 *
 *  2. bxlCmdWriter (const char *buffer, int bufferLen)
 *  		buffer 		: command data buffer.
 *  		bufferLen 	: buffer's length.
 *
 * 	   - Send command data to the printer.
 *	
 *
 *	3. bxlReader (char *buffer, const int timeout)
 *			buffer  : receive status data.
 *			timeout : wait time. 
 *					  if timeout is 0, wait until receive answer.
 *
 *     - Read receive data from the printer.
 *
 *
 *  4. finishBxlAPIMode ()
 *     
 *     - Finish Bixolon CUP API mode.
 *     - This function is not use when use status check command.
 *
 */


#include <stdio.h>
#include <cups/cups.h>
#include "bxlThermalCupsAPI.h"

// CUPS ADD Printer name
//#define PRINTERNAME "BIXOLON_SPP-R300"
#define PRINTERNAME "BIXOLON_SRP-350plusIII"

#define PRINTTEXT 	1
#define RTSTATUS 	0
#define LSBMODE 	0

int printText()
{
	char buffer[2048];

	memset(buffer, 0, sizeof(buffer));

	// Initialize Printer, ESC @
	buffer[0] = 0x1B;
	buffer[1] = 0x40;

	/*
	// Set print mode, ESC !
	buffer[2] = 0x1B;
	buffer[3] = 0x21;
	buffer[4] = 0x00;
	// Select standard mode, ESC S
	buffer[5] = 0x1B;
	buffer[6] = 0x53;
	*/

	if (bxlCmdWriter(buffer, 2) == EXIT_FAILURE) {
		printf("bxlCmdWriter() fail.\n");

		return -1;
	}

	memset(buffer, 0, sizeof(buffer));
	sprintf(buffer, "THIS IS TEST PRINTING AS BIXOLON THERMAL PRINTER API MODE.\xa\xa");

	if (bxlCmdWriter(buffer, strlen(buffer)) == EXIT_FAILURE) {
		printf("bxlCmdWriter() fail.\n");

		return -1;
	}

	if (finishBxlAPIMode() == EXIT_FAILURE) {
		printf("finishBxlAPIMode() fail.\n");

		return -1;
	}

	sleep(1);
	return 0;
}

int checkRealTimeStatus()
{
	char buffer[2048];
	char answer[5];

	memset(answer, 0, sizeof(answer));
	memset(buffer, 0, sizeof(buffer));

	// POS
//	buffer[0] = 0x10;
//	buffer[1] = 0x04;
//	buffer[2] = 0x01;

	// Mobile
	buffer[0] = 0x04;
	buffer[1] = 0x01;

	if (bxlCmdWriter(buffer, 2) == EXIT_FAILURE) {
		printf("bxlCmdWriter() fail.\n");

		return -1;
	}


	/*
	 * int bxlReader (char *buffer, const int timeout);
	 * 		buffer  : receive status data
	 * 		timeout : wait time. 
	 * 				  if timeout is 0, wait until receive answer.
	 */
	if (bxlReader(answer, 0) == EXIT_FAILURE) {
		printf("bxlReader() fail.\n");

		return -1;
	}

	printf("API : receive answer = 0x%x 0x%x 0x%x 0x%x 0x%x\n", 
			answer[0], answer[1], answer[2], answer[3], answer[4]);


	sleep(1);
	return 0;
}

int getLSB()
{
	char buffer[2048];
	char answer[5];
	int i;

	memset(answer, 0, sizeof(answer));
	memset(buffer, 0, sizeof(buffer));

	buffer[0] = 0x1D;
	buffer[1] = 0x61;
//	buffer[2] = 1;	// enable LSB
	buffer[2] = 0; // disable LSB

	if (bxlCmdWriter(buffer, 3) == EXIT_FAILURE) {
		printf("bxlCmdWriter() fail.\n");

		return -1;
	}

	/*
	for (i = 0; i < 5; i++) {
		if (bxlReader(answer, 0) == EXIT_FAILURE) {
			printf("bxlReader() fail.\n");

			return -1;
		}

		printf("API : receive answer = 0x%x 0x%x 0x%x 0x%x 0x%x\n", 
				answer[0], answer[1], answer[2], answer[3], answer[4]);
	}
	*/

	sleep(1);
	return 0;
}

int main (void)
{
	char buffer[2048];
	char answer[5];

	if (startBxlAPIMode(PRINTERNAME) == EXIT_FAILURE) {
		printf("startBxlAPIMode() fail.\n");

		return -1;
	}
	
	memset(buffer, 0, sizeof(buffer));

#if PRINTTEXT
	// print Text sample
	if (printText() == -1) {
		return -1;
	}
#endif

	/*************** CAUTION *****************/
	/*  You must execute bxlStatusChkAgent   */
	/*  before send status check request.    */
	/*****************************************/

#if RTSTATUS
	// <DLE EOT> command
	if (checkRealTimeStatus() == -1) {
		return -1;
	}
#endif

#if LSBMODE
	// <GS a> command
	if (getLSB() == -1) {
		return -1;
	}
#endif


	return 0;
}


